"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

// PLACEHOLDER-ADEIRA-UNIVERSE-INTERNAL


var template = require('@babel/template').default;

var t = require('@babel/types');

var murmurHash = require('@adeira/murmur-hash').default;

var _require = require('@adeira/sx-tailwind'),
    tailwindStyles = _require.tailwindStyles;

var TemplateLiteralHandler = require('./TemplateLiteralHandler').default;

module.exports = function sxTailwindBabelPlugin() {
  var stylesCollector = [];
  var tailwindCallee = '';
  var sxtCallee = '';
  var stylesVarName = 'styles';
  return {
    name: 'sx-tailwind-babel-transform',
    visitor: {
      ImportDeclaration: function ImportDeclaration(path) {
        if (path.node.source.value === '@adeira/sx-tailwind') {
          path.node.specifiers.forEach(function (_ref) {
            var imported = _ref.imported,
                local = _ref.local;

            if (imported.name === 'tailwind') {
              tailwindCallee = local.name;
            } else if (imported.name === 'sxt') {
              sxtCallee = local.name;
            }
          });
          var sxImport = template.ast("import * as sx from '@adeira/sx'");
          path.replaceWith(sxImport);
        }
      },
      CallExpression: function CallExpression(path) {
        if (path.node.callee.name === sxtCallee) {
          path.node.arguments.forEach(function (a) {
            return stylesCollector.push(a.value);
          });
          path.node.callee.name = stylesVarName;
        } else if (path.node.callee.name === tailwindCallee) {
          path.traverse({
            TemplateLiteral: function TemplateLiteral(path) {
              TemplateLiteralHandler(path, stylesCollector, stylesVarName);
            }
          });

          if (path.node.arguments.length === 1 && path.node.arguments[0].type === 'StringLiteral') {
            var _stylesCollector;

            var styles = path.node.arguments[0].value.split(' ').filter(function (s) {
              return s !== '';
            });

            (_stylesCollector = stylesCollector).push.apply(_stylesCollector, (0, _toConsumableArray2.default)(styles));

            path.replaceWith(t.callExpression(t.identifier(stylesVarName), styles.map(function (style) {
              return t.stringLiteral(style);
            })));
          }
        }
      },
      Program: {
        enter: function enter(path, state) {
          stylesCollector = [];
          var filename = state.file.opts.filename.split('/').slice(-2).join('/');
          stylesVarName = "__styles_".concat(murmurHash(filename));
        },
        exit: function exit(path) {
          var declarations = Object.fromEntries(stylesCollector.map(function (style) {
            return [style, tailwindStyles[style]];
          }));

          if (Object.keys(declarations).length > 0) {
            path.node.body.push(template.ast("const ".concat(stylesVarName, " = sx.create(").concat(JSON.stringify(declarations), ")")));
          } // is there SX imported twice?


          var sxImports = path.node.body.reduce(function (a, _ref2, index) {
            var type = _ref2.type,
                source = _ref2.source,
                specifiers = _ref2.specifiers;

            if (type === 'ImportDeclaration' && source.value === '@adeira/sx' && Array.isArray(specifiers) && specifiers[0].type === 'ImportNamespaceSpecifier') {
              a.push(index);
            }

            return a;
          }, []);

          if (sxImports.length > 1) {
            delete path.node.body[sxImports[0]];
          }
        }
      }
    }
  };
};